/*
 * IIR filter coefficient calculation routines.
 *
 * Grisha Sitkarev, <sitkarev@unixkomi.ru>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "macros.h"
#include "common.h"
#include "complex.h"
#include "iir_coef.h"
#include "util.h"

struct iir_filter_coef *
iir_calc_coef(struct transfer_func *z)
{
	struct iir_filter_coef *fc;
	int i, j, n;

	OBJ_NEW(fc);

	/* Calculate how many biquad cascades required. */
	n = (z->np + 1) / 2;

	if (n > 0) {
		fc->casc = xmalloc0(n * sizeof(*fc->casc));
		fc->nrcasc = n;

		/* Calculate numerator coefficients. */

		for (i = j = 0; i < (z->nz / 2); i++, j++) {
			fc->casc[j].b0 = z->h_zero;
			fc->casc[j].b1 = -2.0 * z->zeros[2 * i].re * z->h_zero;
			fc->casc[j].b2 = (z->zeros[2 * i].re * z->zeros[2 * i].re + z->zeros[2 * i].im * z->zeros[2 * i].im) * z->h_zero;
		}

		if (z->nz % 2) {
			fc->casc[j].b0 = z->h_zero;
			fc->casc[j].b1 = -1.0 * z->zeros[z->nz-1].re * z->h_zero;
			fc->casc[j].b2 = 0.0;
		}

		/* Calculate denominator coefficients. */

		for (i = j = 0; i < (z->np / 2); i++, j++) {
			fc->casc[j].a1 = -2.0 * z->poles[2 * i].re;
			fc->casc[j].a2 = z->poles[2 * i].re * z->poles[2 * i].re + z->poles[2 * i].im * z->poles[2 * i].im;
		}

		if (z->nz % 2) {
			fc->casc[j].a1 = -1.0 * z->poles[z->nz - 1].re;
			fc->casc[j].a2 = 0.0;
		}

		return fc;
	}

	return NULL;
}
